<?php

namespace plugin\points_mall\controller\api\auth;

use plugin\points_mall\controller\api\Auth;
use plugin\points_mall\model\PointsMallGoods;
use plugin\points_mall\model\PointsMallGoodsItem;
use plugin\points_mall\model\PointsMallOrderCart;
use plugin\points_mall\service\UserActionService;
use think\admin\helper\QueryHelper;
use think\db\Query;

class Cart extends Auth
{

    /**
     * 获取购买车数据
     * @return void
     */
    public function get()
    {
        PointsMallOrderCart::mQuery(null, function (QueryHelper $query) {
            $query->equal('ghash')->where(['unid' => $this->unid])->with([
                'goods' => static function (Query $query) {
                    $query->with('items');
                },
                'specs' => static function (Query $query) {
                    $query->withoutField('id,create_time,update_time');
                },
            ]);
            $this->success('获取购买车数据！', $query->order('id desc')->page(false, false));
        });
    }

    /**
     * 修改购买车数据
     * @return void
     * @throws \think\db\exception\DbException
     */
    public function set()
    {
        $data = $this->_vali([
            'unid.value'     => $this->unid,
            'ghash.require'  => '商品不能为空！',
            'number.require' => '数量不能为空！',
        ]);
        // 清理数量0的记录
        $map = ['unid' => $this->unid, 'ghash' => $data['ghash']];
        if ($data['number'] < 1) {
            PointsMallOrderCart::mk()->where($map)->delete();
            UserActionService::recount($this->unid);
            $this->success('移除成功！');
        }
        // 检查商品是否存在
        $gspec = PointsMallGoodsItem::mk()->where(['ghash' => $data['ghash']])->findOrEmpty();
        $goods = PointsMallGoods::mk()->where(['code' => $gspec->getAttr('gcode')])->findOrEmpty();
        if ($goods->isEmpty() || $gspec->isEmpty()) $this->error('商品不存在！');
        // 保存商品数据
        $data += ['gcode' => $gspec['gcode'], 'gspec' => $gspec['gspec']];
        if (($cart = PointsMallOrderCart::mk()->where($map)->findOrEmpty())->save($data)) {
            UserActionService::recount($this->unid);
            $this->success('保存成功！', $cart->refresh()->toArray());
        } else {
            $this->error('保存失败！');
        }
    }
}